@@ -529,6 +529,10 @@ class NodeDriver(BaseDriver):
                                before giving up (default is 3)
         @type       max_tries: C{int}
 
+        @keyword    ssh_interface: The interface to wait for. Default is
+                                   'public_ips', other option is 'private_ips'.
+        @type       ssh_interface: C{str}
+
         See L{NodeDriver.create_node} for more keyword args.
 
         >>> from libcloud.compute.drivers.dummy import DummyNodeDriver
@@ -578,16 +582,21 @@ class NodeDriver(BaseDriver):
             password = node.extra.get('password')
 
         try:
-            # Wait until node is up and running and has public IP assigned
-            node = self._wait_until_running(node=node, wait_period=3,
-                                            timeout=NODE_ONLINE_WAIT_TIMEOUT)
+            # Wait until node is up and running and has IP assigned
+            ssh_interface = kwargs.get('ssh_interface', 'public_ips')
+            node, ip_addresses = self._wait_until_running(node=node,
+                    wait_period=3, timeout=NODE_ONLINE_WAIT_TIMEOUT,
+                    ssh_interface=ssh_interface)
+
+            if password:
+                node.extra['password'] = password
 
             ssh_username = kwargs.get('ssh_username', 'root')
             ssh_port = kwargs.get('ssh_port', 22)
             ssh_timeout = kwargs.get('ssh_timeout', 10)
             ssh_key_file = kwargs.get('ssh_key', None)
 
-            ssh_client = SSHClient(hostname=node.public_ips[0],
+            ssh_client = SSHClient(hostname=ip_addresses[0],
                                    port=ssh_port, username=ssh_username,
                                    password=password,
                                    key=ssh_key_file,
@@ -604,10 +613,12 @@ class NodeDriver(BaseDriver):
                                         max_tries=max_tries)
         except Exception:
             e = sys.exc_info()[1]
-            raise DeploymentError(node, e)
+            raise DeploymentError(node=node, original_exception=e, driver=self)
+
         return node
 
-    def _wait_until_running(self, node, wait_period=3, timeout=600):
+    def _wait_until_running(self, node, wait_period=3, timeout=600,
+                            ssh_interface='public_ips'):
         """
         Block until node is fully booted and has an IP address assigned.
 
@@ -622,29 +633,33 @@ class NodeDriver(BaseDriver):
                              (default is 600)
         @type       timeout: C{int}
 
-        @return: C{Node} Node instance on success.
+        @keyword    ssh_interface: The interface to wait for.
+                                   Default is 'public_ips', other option is
+                                   'private_ips'.
+        @type       ssh_interface: C{str}
+
+        @return: C{(Node, ip_addresses)} tuple of Node instance and
+                 list of ip_address on success.
         """
         start = time.time()
         end = start + timeout
 
+        if ssh_interface not in ['public_ips', 'private_ips']:
+            raise ValueError('ssh_interface argument must either be' +
+                             'public_ips or private_ips')
+
         while time.time() < end:
             nodes = self.list_nodes()
             nodes = list([n for n in nodes if n.uuid == node.uuid])
 
-            if len(nodes) == 0:
-                raise LibcloudError(value=('Booted node[%s] ' % node
-                                    + 'is missing from list_nodes.'),
-                                    driver=self)
-
             if len(nodes) > 1:
                 raise LibcloudError(value=('Booted single node[%s], ' % node
                                     + 'but multiple nodes have same UUID'),
                                     driver=self)
 
-            node = nodes[0]
-
-            if (node.public_ips and node.state == NodeState.RUNNING):
-                return node
+            if (len(nodes) == 1 and nodes[0].state == NodeState.RUNNING and \
+                getattr(nodes[0], ssh_interface)):
+                return (nodes[0], getattr(nodes[0], ssh_interface))
             else:
                 time.sleep(wait_period)
                 continue
