@@ -16,15 +16,11 @@
 OpenStack driver
 """
 
-import binascii
-
 try:
     import simplejson as json
 except ImportError:
     import json
 
-import os
-
 import warnings
 
 from itertools import chain
@@ -38,8 +34,9 @@ import base64
 
 from xml.etree import ElementTree as ET
 
-from libcloud.common.openstack import OpenStackBaseConnection, OpenStackDriverMixin
-from libcloud.common.types import MalformedResponseError, LibcloudError
+from libcloud.common.openstack import OpenStackBaseConnection
+from libcloud.common.openstack import OpenStackDriverMixin
+from libcloud.common.types import MalformedResponseError
 from libcloud.compute.types import NodeState, Provider
 from libcloud.compute.base import NodeSize, NodeImage
 from libcloud.compute.base import NodeDriver, Node, NodeLocation
@@ -806,6 +803,16 @@ class OpenStack_1_1_NodeDriver(OpenStackNodeDriver):
         @keyword    ex_files:   File Path => File contents to create on
                                 the node
         @type       ex_files:   C{dict}
+
+        @keyword    ex_keyname:  Name of existing public key to inject into
+                                 instance
+        @type       ex_keyname:  C{string}
+
+        @keyword    ex_userdata: String containing user data
+                                 see
+                                 https://help.ubuntu.com/community/CloudInit
+        @type       ex_userdata: C{string}
+
         """
 
         server_params = self._create_args_to_params(None, **kwargs)
@@ -864,6 +871,13 @@ class OpenStack_1_1_NodeDriver(OpenStackNodeDriver):
                                                                  {}))
         }
 
+        if 'ex_keyname' in kwargs:
+            server_params['key_name'] = kwargs['ex_keyname']
+
+        if 'ex_userdata' in kwargs:
+            server_params['user_data'] = base64.b64encode(
+                b(kwargs['ex_userdata'])).decode('ascii')
+
         if 'name' in kwargs:
             server_params['name'] = kwargs.get('name')
         else:
@@ -1047,6 +1061,7 @@ class OpenStack_1_1_NodeDriver(OpenStackNodeDriver):
                 password=api_node.get('adminPass'),
                 created=api_node['created'],
                 updated=api_node['updated'],
+                key_name=api_node.get('key_name', None),
             ),
         )
 
@@ -1080,3 +1095,17 @@ class OpenStack_1_1_NodeDriver(OpenStackNodeDriver):
         path = urlparse.urlparse(location_header).path
         image_id = path.split('/')[-1]
         return image_id
+
+    def ex_rescue(self, node, password=None):
+        # Requires Rescue Mode extension
+        if password:
+            resp = self._node_action(node, 'rescue', adminPass=password)
+        else:
+            resp = self._node_action(node, 'rescue')
+            password = json.loads(resp.body)['adminPass']
+        node.extra['password'] = password
+        return node
+
+    def ex_unrescue(self, node):
+        resp = self._node_action(node, 'unrescue')
+        return resp.status == httplib.ACCEPTED
