@@ -82,7 +82,7 @@ DRIVERS = {
     Provider.EUCALYPTUS:
         ('libcloud.compute.drivers.ec2', 'EucNodeDriver'),
     Provider.IBM:
-        ('libcloud.compute.drivers.ibm_sbc', 'IBMNodeDriver'),
+        ('libcloud.compute.drivers.ibm_sce', 'IBMNodeDriver'),
     Provider.OPENNEBULA:
         ('libcloud.compute.drivers.opennebula', 'OpenNebulaNodeDriver'),
     Provider.DREAMHOST:
@@ -101,6 +101,8 @@ DRIVERS = {
         ('libcloud.compute.drivers.openstack', 'OpenStackNodeDriver'),
     Provider.NINEFOLD:
         ('libcloud.compute.drivers.ninefold', 'NinefoldNodeDriver'),
+    Provider.VCLOUD:
+        ('libcloud.compute.drivers.vcloud', 'VCloudNodeDriver'),
     Provider.TERREMARK:
         ('libcloud.compute.drivers.vcloud', 'TerremarkDriver'),
     Provider.CLOUDSTACK:
@@ -110,7 +112,11 @@ DRIVERS = {
     Provider.RACKSPACE_NOVA_DFW:
         ('libcloud.compute.drivers.rackspacenova', 'RackspaceNovaDfwNodeDriver'),
     Provider.LIBVIRT:
-        ('libcloud.compute.drivers.libvirt_driver', 'LibvirtNodeDriver')
+        ('libcloud.compute.drivers.libvirt_driver', 'LibvirtNodeDriver'),
+    Provider.JOYENT:
+        ('libcloud.compute.drivers.joyent', 'JoyentNodeDriver'),
+    Provider.VCL:
+        ('libcloud.compute.drivers.vcl', 'VCLNodeDriver')
 }
 
 
