@@ -63,6 +63,8 @@ class Provider(object):
     @cvar CLOUDSIGMA_US: CloudSigma US Las Vegas
     @cvar RACKSPACE_NOVA_BETA: Rackspace Nova Private Beta (ORD)
     @cvar LIBVIRT: Libvirt driver
+    @cvar JOYENT: Joyent driver
+    @cvar VCL: VCL driver
     """
     DUMMY = 0
     EC2 = 1  # deprecated name
@@ -111,6 +113,8 @@ class Provider(object):
     LIBVIRT = 42
     ELASTICHOSTS_US2 = 43
     ELASTICHOSTS_CA1 = 44
+    JOYENT = 45
+    VCL = 46
 
 
 class NodeState(object):
@@ -147,9 +151,17 @@ class DeploymentError(LibcloudError):
 
     @ivar node: L{Node} on which this exception happened, you might want to call L{Node.destroy}
     """
-    def __init__(self, node, original_exception=None):
+    def __init__(self, node, original_exception=None, driver=None):
         self.node = node
         self.value = original_exception
+        self.driver = driver
+
+    def __str__(self):
+        return self.__repr__()
+
+    def __repr__(self):
+        return (('<DeploymentError: node=%s, error=%s, driver=%s>'
+                % (self.node.id, str(self.value), str(self.driver))))
 
 
 """Deprecated alias of L{DeploymentException}"""
