@@ -12,6 +12,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from libcloud.common.openstack import OpenStackDriverMixin
 
 __all__ = [
     'RackspaceUSDNSDriver',
@@ -72,7 +73,6 @@ class RackspaceDNSConnection(OpenStack_1_1_Connection, PollingConnection):
     """
 
     responseCls = RackspaceDNSResponse
-    _url_key = 'dns_url'
     XML_NAMESPACE = None
     poll_interval = 2.5
     timeout = 30
@@ -91,6 +91,33 @@ class RackspaceDNSConnection(OpenStack_1_1_Connection, PollingConnection):
 
         return status == 'COMPLETED'
 
+    def get_endpoint(self):
+        """
+        FIXME:
+        Dirty, dirty hack. DNS doesn't get returned in the auth 1.1 service
+        catalog, so we build it from the servers url.
+        """
+
+        if self._auth_version == "1.1":
+            ep = self.service_catalog.get_endpoint(name="cloudServers")
+
+            return self._construct_dns_endpoint_from_servers_endpoint(ep)
+        elif "2.0" in self._auth_version:
+            ep = self.service_catalog.get_endpoint(name="cloudServers",
+                service_type="compute",
+                region=None)
+
+            return self._construct_dns_endpoint_from_servers_endpoint(ep)
+        else:
+            raise LibcloudError("Auth version %s not supported" % \
+                self._auth_version)
+
+    def _construct_dns_endpoint_from_servers_endpoint(self, ep):
+        if 'publicURL' in ep:
+            return ep['publicURL'].replace("servers", "dns")
+        else:
+            raise LibcloudError('Could not find specified endpoint')
+
 
 class RackspaceUSDNSConnection(RackspaceDNSConnection):
     auth_url = AUTH_URL_US
@@ -100,7 +127,14 @@ class RackspaceUKDNSConnection(RackspaceDNSConnection):
     auth_url = AUTH_URL_UK
 
 
-class RackspaceDNSDriver(DNSDriver):
+class RackspaceDNSDriver(DNSDriver, OpenStackDriverMixin):
+
+    def __init__(self, *args, **kwargs):
+        OpenStackDriverMixin.__init__(self, *args, **kwargs)
+        super(RackspaceDNSDriver, self).__init__(*args, **kwargs)
+
+    def _ex_connection_class_kwargs(self):
+        return self.openstack_connection_kwargs()
 
     RECORD_TYPE_MAP = {
         RecordType.A: 'A',
