@@ -14,6 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import os
 import sys
 import time
 import unittest
@@ -23,6 +24,7 @@ from libcloud.utils.py3 import u
 
 from libcloud.compute.deployment import MultiStepDeployment, Deployment
 from libcloud.compute.deployment import SSHKeyDeployment, ScriptDeployment
+from libcloud.compute.deployment import FileDeployment
 from libcloud.compute.base import Node
 from libcloud.compute.types import NodeState, DeploymentError, LibcloudError
 from libcloud.compute.ssh import BaseSSHClient
@@ -46,7 +48,7 @@ class MockClient(BaseSSHClient):
         self.stderr = ''
         self.exit_status = 0
 
-    def put(self, path, contents, chmod=755):
+    def put(self, path, contents, chmod=755, mode='w'):
         return contents
 
     def run(self, name):
@@ -84,6 +86,15 @@ class DeploymentTests(unittest.TestCase):
         self.assertEqual(self.node, sshd.run(node=self.node,
                         client=MockClient(hostname='localhost')))
 
+    def test_file_deployment(self):
+        # use this file (__file__) for obtaining permissions
+        target = os.path.join('/tmp', os.path.basename(__file__))
+        fd = FileDeployment(__file__, target)
+        self.assertEqual(target, fd.target)
+        self.assertEqual(__file__, fd.source)
+        self.assertEqual(self.node, fd.run(
+                node=self.node, client=MockClient(hostname='localhost')))
+
     def test_script_deployment(self):
         sd1 = ScriptDeployment(script='foobar', delete=True)
         sd2 = ScriptDeployment(script='foobar', delete=False)
@@ -128,15 +139,33 @@ class DeploymentTests(unittest.TestCase):
             self.fail('TypeError was not thrown')
 
     def test_wait_until_running_running_instantly(self):
-        node2 = self.driver._wait_until_running(node=self.node, wait_period=1,
-                                                timeout=10)
+        node2, ips = self.driver._wait_until_running(node=self.node, wait_period=1,
+                                                     timeout=10)
         self.assertEqual(self.node.uuid, node2.uuid)
+        self.assertEqual(['67.23.21.33'], ips)
 
     def test_wait_until_running_running_after_1_second(self):
         RackspaceMockHttp.type = '1_SECOND_DELAY'
-        node2 = self.driver._wait_until_running(node=self.node, wait_period=1,
-                                                timeout=10)
+        node2, ips = self.driver._wait_until_running(node=self.node, wait_period=1,
+                                                     timeout=10)
         self.assertEqual(self.node.uuid, node2.uuid)
+        self.assertEqual(['67.23.21.33'], ips)
+
+    def test_wait_until_running_running_after_1_second_private_ips(self):
+        RackspaceMockHttp.type = '1_SECOND_DELAY'
+        node2, ips = self.driver._wait_until_running(node=self.node, wait_period=1,
+                                                     timeout=10, ssh_interface='private_ips')
+        self.assertEqual(self.node.uuid, node2.uuid)
+        self.assertEqual(['10.176.168.218'], ips)
+
+    def test_wait_until_running_invalid_ssh_interface_argument(self):
+        try:
+            self.driver._wait_until_running(node=self.node, wait_period=1,
+                                            ssh_interface='invalid')
+        except ValueError:
+            pass
+        else:
+            self.fail('Exception was not thrown')
 
     def test_wait_until_running_timeout(self):
         RackspaceMockHttp.type = 'TIMEOUT'
@@ -158,7 +187,7 @@ class DeploymentTests(unittest.TestCase):
                                             timeout=1)
         except LibcloudError:
             e = sys.exc_info()[1]
-            self.assertTrue(e.value.find('is missing from list_nodes') != -1)
+            self.assertTrue(e.value.find('Timed out after 1 second') != -1)
         else:
             self.fail('Exception was not thrown')
 
