@@ -97,6 +97,14 @@ class OpenStack_1_0_Tests(unittest.TestCase, TestCaseMixin):
         self.driver.connection._populate_hosts_and_request_paths()
         clear_pricing_data()
 
+    def test_auth_token_is_set(self):
+        self.driver.connection._populate_hosts_and_request_paths()
+        self.assertEquals(self.driver.connection.auth_token, "603d2bd9-f45c-4583-b91c-2c8eac0b5654")
+
+    def test_auth_token_expires_is_set(self):
+        self.driver.connection._populate_hosts_and_request_paths()
+        self.assertEquals(self.driver.connection.auth_token_expires, "2011-09-18T02:44:17.000-05:00")
+
     def test_auth(self):
         OpenStackMockHttp.type = 'UNAUTHORIZED'
         try:
@@ -541,6 +549,24 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
         clear_pricing_data()
         self.node = self.driver.list_nodes()[1]
 
+    def test_auth_token_is_set(self):
+        # change base url and trash the current auth token so we can re-authenticate
+        self.driver.connection._ex_force_base_url = 'http://ex_force_base_url.com:666/forced_url'
+        self.driver.connection.auth_token = None
+        self.driver.connection.auth_token_expires = None
+        self.driver.connection._populate_hosts_and_request_paths()
+
+        self.assertEquals(self.driver.connection.auth_token, "aaaaaaaaaaaa-bbb-cccccccccccccc")
+
+    def test_auth_token_expires_is_set(self):
+        # change base url and trash the current auth token so we can re-authenticate
+        self.driver.connection._ex_force_base_url = 'http://ex_force_base_url.com:666/forced_url'
+        self.driver.connection.auth_token = None
+        self.driver.connection.auth_token_expires = None
+        self.driver.connection._populate_hosts_and_request_paths()
+
+        self.assertEquals(self.driver.connection.auth_token_expires, "2011-11-23T21:00:14.000-06:00")
+
     def test_ex_force_base_url(self):
         # change base url and trash the current auth token so we can re-authenticate
         self.driver.connection._ex_force_base_url = 'http://ex_force_base_url.com:666/forced_url'
@@ -673,6 +699,18 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
         self.assertEqual(node.extra['password'], 'racktestvJq7d3')
         self.assertEqual(node.extra['metadata']['My Server Name'], 'Apache1')
 
+    def test_create_node_with_ex_keyname_and_ex_userdata(self):
+        image = NodeImage(id=11, name='Ubuntu 8.10 (intrepid)', driver=self.driver)
+        size = NodeSize(1, '256 slice', None, None, None, None, driver=self.driver)
+        node = self.driver.create_node(name='racktest', image=image, size=size,
+                                       ex_keyname='devstack',
+                                       ex_userdata='sample data')
+        self.assertEqual(node.id, '26f7fbee-8ce1-4c28-887a-bfe8e4bb10fe')
+        self.assertEqual(node.name, 'racktest')
+        self.assertEqual(node.extra['password'], 'racktestvJq7d3')
+        self.assertEqual(node.extra['metadata']['My Server Name'], 'Apache1')
+        self.assertEqual(node.extra['key_name'], 'devstack')
+
     def test_destroy_node(self):
         self.assertTrue(self.node.destroy())
 
@@ -802,6 +840,24 @@ class OpenStack_1_1_Tests(unittest.TestCase, TestCaseMixin):
         params = [("key", "value1"), ("key", "value2") ]
         self.driver.connection.request("/servers/12067", params=params)
 
+    def test_ex_rescue_with_password(self):
+        node = Node(id=12064, name=None, state=None, public_ips=None,
+                    private_ips=None, driver=self.driver)
+        n = self.driver.ex_rescue(node, 'foo')
+        self.assertEqual(n.extra['password'], 'foo')
+
+    def test_ex_rescue_no_password(self):
+        node = Node(id=12064, name=None, state=None, public_ips=None,
+                    private_ips=None, driver=self.driver)
+        n = self.driver.ex_rescue(node)
+        self.assertEqual(n.extra['password'], 'foo')
+
+    def test_ex_unrescue(self):
+        node = Node(id=12064, name=None, state=None, public_ips=None,
+                    private_ips=None, driver=self.driver)
+        result = self.driver.ex_unrescue(node)
+        self.assertTrue(result)
+
 
 class OpenStack_1_1_FactoryMethodTests(OpenStack_1_1_Tests):
     should_list_locations = False
@@ -871,6 +927,9 @@ class OpenStack_1_1_MockHttp(MockHttpTestCase):
             return (httplib.ACCEPTED, "",
                     {"location": "http://127.0.0.1/v1.1/68/images/4949f9ee-2421-4c81-8b49-13119446008b"},
                     httplib.responses[httplib.ACCEPTED])
+        elif "rescue" in json.loads(body):
+            return (httplib.OK, '{"adminPass": "foo"}', {},
+                    httplib.responses[httplib.OK])
 
         return (httplib.ACCEPTED, "", {}, httplib.responses[httplib.ACCEPTED])
 
