@@ -37,6 +37,41 @@ class RackspaceUSTests(unittest.TestCase):
         RackspaceMockHttp.type = None
         self.driver = self.klass(*DNS_PARAMS_RACKSPACE)
         self.driver.connection.poll_interval = 0.0
+        # normally authentication happens lazily, but we force it here
+        self.driver.connection._populate_hosts_and_request_paths()
+
+    def test_force_auth_token_kwargs(self):
+        kwargs = {
+            'ex_force_auth_token': 'some-auth-token',
+            'ex_force_base_url': 'https://dns.api.rackspacecloud.com/v1.0/11111'
+        }
+        driver = self.klass(*DNS_PARAMS_RACKSPACE, **kwargs)
+        driver.list_zones()
+
+        self.assertEquals(kwargs['ex_force_auth_token'],
+            driver.connection.auth_token)
+        self.assertEquals('/v1.0/11111',
+            driver.connection.request_path)
+
+    def test_force_auth_url_kwargs(self):
+        kwargs = {
+            'ex_force_auth_version': '2.0',
+            'ex_force_auth_url': 'https://identity.api.rackspace.com'
+        }
+        driver = self.klass(*DNS_PARAMS_RACKSPACE, **kwargs)
+
+        self.assertEquals(kwargs['ex_force_auth_url'],
+            driver.connection._ex_force_auth_url)
+        self.assertEquals(kwargs['ex_force_auth_version'],
+            driver.connection._auth_version)
+
+    def test_gets_auth_2_0_endpoint(self):
+        kwargs = {'ex_force_auth_version': '2.0_password'}
+        driver = self.klass(*DNS_PARAMS_RACKSPACE, **kwargs)
+        driver.connection._populate_hosts_and_request_paths()
+
+        self.assertEquals('https://dns.api.rackspacecloud.com/v1.0/11111',
+            driver.connection.get_endpoint())
 
     def test_list_record_types(self):
         record_types = self.driver.list_record_types()
@@ -270,9 +305,10 @@ class RackspaceMockHttp(MockHttp):
     fixtures = DNSFileFixtures('rackspace')
     base_headers = {'content-type': 'application/json'}
 
-    # fake auth token response
+
     def _v1_1_auth(self, method, url, body, headers):
         body = self.fixtures.load('auth_1_1.json')
+        # fake auth token response
         headers = {'content-length': '657', 'vary': 'Accept,Accept-Encoding',
                    'server': 'Apache/2.2.13 (Red Hat)',
                    'connection': 'Keep-Alive',
@@ -281,6 +317,14 @@ class RackspaceMockHttp(MockHttp):
         return (httplib.OK, body, headers,
                 httplib.responses[httplib.OK])
 
+    def _v2_0_tokens(self, method, url, body, headers):
+        body = self.fixtures.load('auth_2_0.json')
+        headers = {
+            'content-type': 'application/json'
+        }
+        return (httplib.OK, body, headers,
+                httplib.responses[httplib.OK])
+
     def _v1_0_11111_domains(self, method, url, body, headers):
         body = self.fixtures.load('list_zones_success.json')
         return (httplib.OK, body, self.base_headers,
